//
// LocoTool - A program to edit Locomotion object description .dat files
//
// Copyright (C) 2004 by Josef Drexler
//
// This file is distributed under the term of the GNU General Public Licence
// version 2 or higher.  Please read the file LICENCE for details.
//
//
// objects.c - object class-specific data structure definitions
//

#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "locodef.h"

//
// Two main contents of this file:
// - the object class-specific data structures
//   (in varinf arrays plsu char* arrays for the bit fields)
// - the object class-specific item descriptions
//
// Each varinf entry is either a simple variable, a sub-structure, a
// bit field, or an array of those.  All must be in sequence without
// holes.
//
// The descriptions tell LocoTool what kind of data is arranged in which
// order in the .dat file.
//
//
// All object classes are then put into the objclasses[] array at the bottom
// of this file.
//

// ***********************
//      General items
// ***********************

	// the known bits in the flags of each sprite
const char *spriteflags[] = {
	"hasdata", "", "chunked", "",
	"", "", "copy", "",
	NULL
};

	// if a variable etc. can have different names, enter the aliases here
const char* namealiases[][2] = {
//	{ "oldname", "newname" },
	{ "wideslope", "steepslope" },
	{ NULL },
};

// sometimes the number of entries of an item is not simply constant or
// stored somewhere; in this case these handlers return the calculated number
// see also the comment in en_desctype in locodef.h
int vehnumtrack(uint8_t *data)
{
  return (data[2] < 2) && !(data[0xE1] & 2) ? 0 : -1;
}

int vehnumrackrail(uint8_t *data)
{
  return (data[2] < 2) && (data[0xE0]&0x40) ? 0 : -1;
}

typedef int calcdescnumproc(uint8_t *);

calcdescnumproc *calcdescnum[] = {
	vehnumtrack, vehnumrackrail,
};

int getnum(uint8_t *data, uint32_t *ofs, int32_t numdef)
{
  int8_t type, arg;
  int16_t num;

  type = numdef >> 24;
  arg = (numdef & 0xff0000) >> 16;
  num = numdef & 0xffff;

  switch (type) {
	case 0:
		return num;

	case 1:
		if (data[*ofs] != 0xff)
			return 0xffffff;

		(*ofs)++;
		return -1;

	case 2:
		num = data[-num] & arg;
		if (num)
			return num;
		else
			return -1;

	case 3:
		return data[-num] ? 0 : -1;

	case 4:
		return calcdescnum[num](data);

	case -1:
		if (num < 0) {
			num = data[-num];
			if (num)
				return num;
			else
				return -1;
		}

		// else fall through

  }

  die("Invalid description count %x", numdef);
}


// ***********************
//  SIMPLE CLASS HANDLER
// ***********************

varinf simplevars[] = {
	{ 0x00, 1, 6, "" },
	{ -1 },
};

objdesc simpledesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_sprites },
	{ desc_END },
};

objdesc simpledescnosprites[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_END },
};



// ***********************
// Class 00:  INTERFACES
// ***********************

varinf interfacevars[] = {
	{ 0x00, 1, 24, "" },
	{ -1 },
};


// ***********************
// Class 01:  SOUNDS
// ***********************

varinf sfxvars[] = {
	{ 0x00, 1, 8, "" },
	{ 0x08,-4, 1, "" },
	{ -1 },
};

objdesc sfxdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_sounds },
	{ desc_END },
};


// ***********************
// Class 02:  CURRENCIES
// ***********************

varinf currvars[] = {
	{ 0x00, 1, 10, "" },
	{ 0x0A, 1, 1, "zeroes" },
	{ 0x0B, 1, 1, "shiftnum" },
	{ -1 },
};

objdesc currdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_lang, { 1 } },
	{ desc_lang, { 2 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 03:  EXHAUST FX
// ***********************

varinf exhfxvars[] = {
	{ 0x00, 1, 30, "" },
	{ 0x1E, 1, 1, "numsnd" },
	{ 0x1F, 1, 9, "" },
	{ -1 },
};

auxdesc exhfxaux[] = {
	{ "", NULL },
	{ "", NULL },
	{ NULL }
};

objdesc exhfxdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_auxdatavar, { 0, 0, 2, 1 } },
	{ desc_auxdatavar, { 1, 0, 2, 1 } },
	{ desc_useobj, { -0x1e, ob_soundeffect, 0x01, -1 } },
	{ desc_sprites },
	{ desc_END },
};



// ***********************
// Class 04:  CLIFF FACES
// ***********************

// (see simple vars above)


// ***********************
// Class 05:   WATER
// ***********************

varinf watervars[] = {
	{ 0x00, 1, 14, "" },
	{ -1 },
};


// ***********************
// Class 06:  GROUND
// ***********************

varinf groundvars[] = {
	{ 0x00, 1, 30, "" },
	{ -1 },
};

objdesc grounddesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_useobj, { 0, ob_cliff, 0x04, -1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 07:  TOWN NAMES
// ***********************

varinf townpart[] = {
	{ 0x00, 1, 1, "num" },
	{ 0x01, 1, 1, "numempty" },
	{ 0x02, 2, 1, "indexofs" },
	{ -1 },
};

varinf townvars[] = {
	{ 0x00, 1, 2, "" },
	{ 0x02, 4, 6, "part", townpart },
	{ -1 },
};

objdesc towndesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_strtable, { 0, 2+0*4, 4+0*4 } },
	{ desc_strtable, { 1, 2+1*4, 4+1*4 } },
	{ desc_strtable, { 2, 2+2*4, 4+2*4 } },
	{ desc_strtable, { 3, 2+3*4, 4+3*4 } },
	{ desc_strtable, { 4, 2+4*4, 4+4*4 } },
	{ desc_strtable, { 5, 2+5*4, 4+5*4 } },
	{ desc_END },
};


// ***********************
// Class 08:  CARGO TYPES
// ***********************

const char *cargoflags[] = {
	"", "refitoption",
	NULL,
};

varinf cargovars[] = {
	{ 0x00, 1, 2, "" },
	{ 0x02, 2, 1, "unitweight" },
	{ 0x04, 1, 12, "" },
	{ 0x10, 1, 1, "id" },
	{ 0x11, 1, 1, "" },
	{ 0x12, 1, 1, "flags", NULL, cargoflags },
	{ 0x13, 1, 2, "" },
	{ 0x15, 1, 1, "peakdays" },
	{ 0x16, 1, 1, "decay1days" },
	{ 0x17, 2, 1, "decay1rate" },
	{ 0x19, 2, 1, "decay2rate" },
	{ 0x1B, 2, 1, "paymentfactor" },
	{ 0x1D, 1, 1, "paymentind" },
	{ 0x1E, 1, 1, "" },
	{ -1 },
};

objdesc cargodesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_lang, { 1 } },
	{ desc_lang, { 2 } },
	{ desc_lang, { 3 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 09:    FENCES
// ***********************

const char *fenceflags[] = {
	NULL
};

varinf fencevars[] = {
	{ 0x00, 1, 7, "" },
	{ 0x07, 1, 1, "flags", NULL, fenceflags },
	{ 0x08, 1, 1, "" },
	{ 0x09, 1, 1, "" },
	{ -1 },
};


// ***********************
// Class 0A:    SIGNALS
// ***********************

varinf signalvars[] = {
	{ 0x00, 1, 30, "" },
	{ -1 },
};

objdesc signaldesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_lang, { 1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 0B:  CROSSINGS
// ***********************

varinf crossingvars[] = {
	{ 0x00, 1, 18, "" },
	{ -1 },
};


// ***********************
// Class 0C: STREET LIGHTS
// ***********************

varinf lightvars[] = {
	{ 0x00, 1, 12, "" },
	{ -1 },
};


// ***********************
// Class 0D:    TUNNELS
// ***********************

// (see simplevars above)


// ***********************
// Class 0E:   BRIDGES
// ***********************

varinf bridgevars[] = {
	{ 0x00, 1, 2, "" },
	{ 0x02, 1, 1, "noroof" },
	{ 0x03, 1, 1, "" },
	{ 0x04, 1, 1, "" },
	{ 0x05, 1, 1, "" },
	{ 0x06, 1, 1, "" },
	{ 0x07, 1, 1, "" },
	{ 0x08, 1, 1, "spanlength" },
	{ 0x09, 1, 1, "pillarspacing" },
	{ 0x0A, 2, 1, "maxspeed" },
	{ 0x0C, 1, 1, "maxheight" },
	{ 0x0D, 1, 1, "costind" },
	{ 0x0E,-2, 1, "basecostfact" },
	{ 0x10,-2, 1, "heightcostfact" },
	{ 0x12,-2, 1, "salecostfact" },
	{ 0x14, 2, 1, "disabledtrackcfg" },
	{ 0x16, 1,22, "" },
	{ -1 },
};


// ***********************
// Class 0F:  TRAIN STATIONS
// ***********************

varinf trnstatvars[] = {
	{ 0x00, 1, 34, "" },
	{ 0x22, 1, 1, "numcompat" },
	{ 0x23, 1, 7, "" },
	{ 0x2a, 1, 2, "designed" },
	{ 0x2c, 1, 2, "obsolete" },
	{ 0x2e, 1, 128, "" },
	{ -1 },
};

auxdesc trnstataux[] = {
	{ "", NULL },
	{ NULL },
};

objdesc trnstatdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_auxdatavar, { 0, 32, 1, 4 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 10:  TRACK MODS
// ***********************

const char *track_curvetypes[] = {
	"", "widecurve", "mediumcurve", "smallcurve",
	"tightcurve", "normalslope", "steepslope",
	NULL,
};

varinf trkmodvars[] = {
	{ 0x00, 1, 2, "" },
	{ 0x02, 2, 1, "curvetypes", NULL, track_curvetypes },
	{ 0x04, 1, 14, "" },
	{ -1 },
};


// ***********************
// Class 11:   TRACKS
// ***********************

varinf trackvars[] = {
	{ 0x00, 1, 2, "" },
	{ 0x02, 1, 1, "curvetypes", NULL, track_curvetypes },
	{ 0x03, 1, 4, "" },
	{ 0x07, 1, 1, "numcompat" },
	{ 0x08, 1, 1, "nummods" },
	{ 0x09, 1, 1, "numsignals" },
	{ 0x0A, 1, 10, "" },
	{ 0x14,-2, 1, "buildcostfact" },
	{ 0x16,-2, 1, "sellcostfact" },
	{ 0x18,-2, 1, "tunnelcostfact" },
	{ 0x1A, 1, 1, "costind" },
	{ 0x1B, 1, 1, "" },
	{ 0x1C, 2, 1, "curvespeed" },
	{ 0x1E, 1, 6, "" },
	{ 0x24, 1, 1, "numbridges" },
	{ 0x25, 1, 7, "" },
	{ 0x2C, 1, 1, "numstations" },
	{ 0x2D, 1, 7, "" },
	{ 0x34, 1, 1, "displayoffset" },
	{ 0x35, 1, 1, "" },
	{ -1 },
};

objdesc trackdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_useobj, { -7, ob_compatible, 0x11, 0x14, -1 } },
	{ desc_useobj, { -8, ob_trackmod, 0x10, -1 } },
	{ desc_useobj, { -9, ob_signal, 0x0A, -1 } },
	{ desc_useobj, {  0, ob_tunnel, 0x0D, -1 } },
	{ desc_useobj, { -0x24, ob_bridge, 0x0E, -1 } },
	{ desc_useobj, { -0x2C, ob_station, 0x0F, -1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 12:  ROAD STATIONS
// ***********************

varinf roadstvars[] = {
	{ 0x00, 1, 33, "" },
	{ 0x21, 1, 1, "numcompat" },
	{ 0x22, 1, 6, "" },
	{ 0x28, 2, 1, "designed" },
	{ 0x2a, 2, 1, "obsolete" },
	{ 0x2c, 1, 66, "" },
	{ -1 },
};

auxdesc roadstaux[] = {
	{ "", NULL },
	{ NULL },
};

objdesc roadstdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_useobj, { 0, ob_cargo, 0x08, -1 } },
	{ desc_auxdatavar, { 0, 16, 1, 4 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 13:  ROAD MODS
// ***********************

// (same as track mods)


// ***********************
// Class 14:    ROADS
// ***********************

varinf roadvars[] = {
	{ 0x00, 1, 20, "" },
	{ 0x14, 1, 1, "numbridges" },
	{ 0x15, 1, 8, "" },
	{ 0x1D, 1, 1, "numstations" },
	{ 0x1E, 1, 10, "" },
	{ 0x28, 1, 1, "numcompat" },
	{ 0x29, 1, 7, "" },
	{ -1 },
};

objdesc roaddesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_useobj, { -40, ob_compatible, 0x11, 0x14, -1 } },
	{ desc_useobj, { -37, ob_roadmod, 0x13, -1 } },
	{ desc_useobj, {  0, ob_tunnel, 0x0D, -1 } },
	{ desc_useobj, { -20, ob_bridge, 0x0E, -1 } },
	{ desc_useobj, { -28, ob_station, 0x12, -1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 15:   AIRPORTS
// ***********************

varinf airportvars[] = {
	{ 0x00, 1, 16, "" },
	{ 0x10, 2, 1, "allowedplanetypes" },
	{ 0x12, 1, 1, "numdata" },
	{ 0x13, 1, 149, "" },
	{ 0xA8, 2, 1, "designed" },
	{ 0xAA, 2, 1, "obsolete" },
	{ 0xAC, 1, 14, "" },
	{ -1 },
};

auxdesc airportaux[] = {
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ NULL },
};

objdesc airportdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_auxdata, { 0, 0, 1, -0x12 } },
	{ desc_auxdata, { 1, 0, -2, -0x12 } },
	{ desc_auxdatavar, { 2, -0x13, 1, 1 } },
	{ desc_auxdatavar, { 3, 0, -4, 1 } },
	{ desc_auxdata, { 4, -0xac, 1, 8 } },
	{ desc_auxdata, { 5, -0xad, 1, 12 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 16:     DOCKS
// ***********************

varinf dockvars[] = {
	{ 0x00, 1, 18, "" },
	{ 0x12, 1, 1, "numdata" },
	{ 0x13, 1, 13, "" },
	{ 0x20, 2, 1, "designed" },
	{ 0x22, 2, 1, "obsolete" },
	{ 0x24, 1, 4, "" },
	{ -1 },
};

auxdesc dockaux[] = {
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ NULL },
};

objdesc dockdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_auxdata, { 0, 0, 1, -0x12 } },
	{ desc_auxdata, { 1, 0, -2, -0x12 } },
	{ desc_auxdatavar, { 2, -0x13, 1, 1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 17:   VEHICLES
// ***********************


	// the known bits in the vehicle sprite structure flags
const char *vehspriteflags[] = {
	"hassprites", "", "", "", "",
	"reversed",
	NULL
};

	// the vehicle sprite definition structure
varinf vehsprites[] = {
	{ 0x00, 1, 1, "numdir" },
	{ 0x01, 1, 1, "" },
	{ 0x02, 1, 1, "" },
	{ 0x03, 1, 1, "vehtype" },
	{ 0x04, 1, 1, "numunits" },
	{ 0x05, 1, 1, "" },
	{ 0x06, 1, 1, "bogeypos" },
	{ 0x07, 1, 1, "flags", NULL, vehspriteflags },
	{ 0x08, 1, 1, "" },
	{ 0x09, 1, 1, "" },
	{ 0x0A, 1, 1, "" },
	{ 0x0B, 1, 1, "" },
	{ 0x0C, 1, 1, "" },
	{ 0x0D, 1, 1, "" },
	{ 0x0E, 1, 1, "spritenum" },
	{ 0x0F, 1, 3, "" },
	{ 0x12, 4, 1, "" },
	{ 0x16, 4, 1, "" },
	{ 0x1A, 4, 1, "" },
	{ -1 },
};

	// a vehicle structure whose purpose is mostly unknown
varinf vehunk[] = {
	{ 0x00, 1, 1, "length" },
	{ 0x01, 1, 1, "" },
	{ 0x02, 1, 1, "" },
	{ 0x03, 1, 1, "" },
	{ 0x04, 1, 1, "spriteind" },
	{ 0x05, 1, 1, "" },
	{ -1 },
};

	// the vehicle flags
const char *vehflags[] = {
	"", "", "", "",
	"", "", "rackrail", "",
	"", "anytrack", "", "cancouple",
	"dualhead", "", "refittable", "noannounce",
	NULL
};

	// the main vehicle data structure
varinf vehvars[] = {
	{ 0x000, 1, 2, "" },
	{ 0x002, 1, 1, "class" },
	{ 0x003, 1, 1, "type" },
	{ 0x004, 1, 1, "" },
	{ 0x005, 1, 1, "" },
	{ 0x006, 1, 1, "nummods" },
	{ 0x007, 1, 1, "costind" },
	{ 0x008,-2, 1, "costfact" },	// size -2 means signed word
	{ 0x00A, 1, 1, "reliability" },
	{ 0x00B, 1, 1, "runcostind" },
	{ 0x00C,-2, 1, "runcostfact" },
	{ 0x00E, 1, 1, "colourtype" },
	{ 0x00F, 1, 1, "numcompat" },
	{ 0x010, 1,20, "" },
	{ 0x024, 6, 4, "", vehunk },
	{ 0x03C,30, 4, "sprites", vehsprites },
	{ 0x0B4, 1,36, "" },
	{ 0x0D8, 2, 1, "power" },
	{ 0x0DA, 2, 1, "speed" },
	{ 0x0DC, 2, 1, "rackspeed" },
	{ 0x0DE, 2, 1, "weight" },
	{ 0x0E0, 2, 1, "flags", NULL, vehflags },
	{ 0x0E2, 1,44, "" },
	{ 0x10E, 1, 1, "visfxheight" },
	{ 0x10F, 1, 1, "visfxtype" },
	{ 0x110, 1, 1, "" },
	{ 0x111, 1, 1, "" },
	{ 0x112, 1, 1, "wakefxtype" },
	{ 0x113, 1, 1, "" },
	{ 0x114, 2, 1, "designed" },
	{ 0x116, 2, 1, "obsolete" },
	{ 0x118, 1, 1, "" },
	{ 0x119, 1, 1, "startsndtype" },
	{ 0x11A, 1,64, "" },
	{ 0x15A, 1, 1, "numsnd" },
	{ 0x15B, 1, 3, "" },
	{ -1 },
};

objdesc vehdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_useobj, { descnumspec(0), ob_tracktype, 0x11, 0x14, -1 } },
	{ desc_useobj, { -6, ob_trackmod, 0x10, 0x13, -1 } },
	{ desc_cargo, { 2 } },
	{ desc_useobj, { descnumif(0x10f), ob_visualeffect, 0x03, -1 } },
	{ desc_useobj, { descnumif(0x112), ob_wakeeffect, 0x03, -1 } },
	{ desc_useobj, { descnumspec(1), ob_rackrail, 0x10, -1 } },
	{ desc_useobj, { -15, ob_compatible, 0x17, -1 } },
	{ desc_useobj, { descnumif(0x119), ob_startsnd, 0x01, -1 } },
	{ desc_useobj, { descnumand(0x15a,0x7f), ob_soundeffect, 0x01, -1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 18:    TREES
// ***********************

varinf treevars[] = {
	{ 0x00, 1, 3, "" },
	{ 0x03, 1, 1, "height" },
	{ 0x04, 1, 72, "" },
	{ -1 },
};


// ***********************
// Class 19:    SNOW
// ***********************

// (see simple vars above)


// ***********************
// Class 1A:   CLIMATES
// ***********************

varinf climvars[] = {
	{ 0x00, 1, 10, "" },
	{ -1 },
};


// ***********************
// Class 1B:   SHAPES
// ***********************

varinf shapevars[] = {
	{ 0x00, 1, 14, "" },
	{ -1 },
};


// ***********************
// Class 1C:   BUILDINGS
// ***********************

varinf bldngvars[] = {
	{ 0x00, 1, 148, "" },
	{ 0x94, 2, 1, "earliestyr" },
	{ 0x96, 2, 1, "latestyr" },
	{ 0x98, 1, 38, "" },
	{ -1 },
};

auxdesc bldngaux[] = {
	{ "", NULL },
	{ "", NULL },
	{ NULL },
};

objdesc bldngdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_auxdatavar, { 0, -7, 1, 1 } },
	{ desc_useobj, { 4, ob_cargo, 0x08, 0xFF, -1 } },
	{ desc_auxdatafix, { 1, -0xad, 1, 2 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 1D:  SCAFFOLDING
// ***********************

varinf scaffvars[] = {
	{ 0x00, 1, 18, "" },
	{ -1 },
};


// ***********************
// Class 1E:  INDUSTRIES
// ***********************

const char *industryflags[] = {
	"", "", "", "",		// 1, 2, 4, 8
	"", "", "", "",		// 10, 20, 40, 80
	"", "", "", "",		// 100, 200, 400, 800
	"", "", "", "",		// 1000, 2000, 4000, 8000
	"", "needall", "canincreaseproduction", "candecreaseproduction",
				// 100000, 200000, 400000, 800000
	NULL,
};

varinf indvars[] = {
	{ 0x00, 1, 202, "" },
	{ 0xCA, 2, 1, "firstyear" },
	{ 0xCC, 2, 1, "lastyear" },
	{ 0xCE, 1, 1, "" },
	{ 0xCF, 1, 1, "costind" },
	{ 0xD0, 2, 1, "costfactor1" },
	{ 0xD2, 1, 18, "" },
	{ 0xE4, 4, 1, "flags", NULL, industryflags },
	{ 0xE8, 1, 12, "" },
	{ -1 },
};

auxdesc indaux[] = {
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ "", NULL },
	{ NULL },
};

objdesc inddesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_lang, { 1 } },
	{ desc_lang, { 2 } },
	{ desc_lang, { 3 } },
	{ desc_lang, { 4 } },
	{ desc_lang, { 5 } },
	{ desc_lang, { 6 } },
	{ desc_lang, { 7 } },
	{ desc_auxdata, { 0, 0, 1, -30 } },
	{ desc_auxdata, { 1, 0, 2, -30 } },
	{ desc_auxdata, { 2, 0, 1, 0, 4 } },
	{ desc_auxdatavar, { 3, 0, 2, 1 } },
	{ desc_auxdatavar, { 4, -31, 1, 1 } },
	{ desc_auxdata, { 5, 0, 1, -0xbd } },
	{ desc_useobj, { 2, ob_produces, 0x08, 0xFF, -1 } },
	{ desc_useobj, { 3, ob_accepts, 0x08, 0xFF, -1 } },
	{ desc_useobj, { 6, ob_fence, 0x09, 0xFF, -1 } },
	{ desc_auxdata, { 1, -0xad, 1, 2 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 1F:   REGIONS
// ***********************

varinf regionvars[] = {
	{ 0x00, 1, 18, "" },
	{ -1 },
};

objdesc regiondesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_useobj, { -8, ob_cargo, 0x08, -1 } },
	{ desc_useobj, { descnumuntil(), ob_default, 0x100, -1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 20:  COMPANIES
// ***********************

varinf compvars[] = {
	{ 0x00, 1, 56, "" },
	{ -1 },
};

objdesc compdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_lang, { 1 } },
	{ desc_sprites },
	{ desc_END },
};


// ***********************
// Class 21:   TEXTS
// ***********************

// (uses simple vars)

objdesc textdesc[] = {
	{ desc_objdata },
	{ desc_lang, { 0 } },
	{ desc_lang, { 1 } },
	{ desc_END },
};


// ***********************
//  END OF CLASS HANDLERS
// ***********************

//
// Array defining object class-specific data structures and descriptions
//
// Entries are object structure description, its size, auxdata names, description
//
objclass objclasses[NUMOBJCLASSES] = {
	{ interfacevars, 24, NULL, simpledesc },	// 00 Interfaces
	{ sfxvars, 12, NULL, sfxdesc },			// 01 Sound effects
	{ currvars, 12, NULL, currdesc },		// 02 Currencies
	{ exhfxvars, 40, exhfxaux, exhfxdesc },		// 03 Exhaust effects
	{ simplevars, 6, NULL, simpledesc },		// 04 Cliff faces
	{ watervars, 14, NULL, simpledesc },		// 05 Water
	{ groundvars, 30, NULL, grounddesc },		// 06 Ground
	{ townvars, 26, NULL, towndesc },		// 07 Town names
	{ cargovars, 31, NULL, cargodesc },		// 08 Cargos
	{ fencevars, 10, NULL, simpledesc },		// 09 Fences
	{ signalvars, 30, NULL, signaldesc },		// 0A Signals
	{ crossingvars, 18, NULL, simpledesc },		// 0B Crossings
	{ lightvars, 12, NULL, simpledesc },		// 0C Street lights
	{ simplevars, 6, NULL, simpledesc },		// 0D Tunnels
	{ bridgevars, 44, NULL, simpledesc },		// 0E Bridges
	{ trnstatvars, 174, trnstataux, trnstatdesc },	// 0F Train stations
	{ trkmodvars, 18, NULL, simpledesc },		// 10 Track modifications
	{ trackvars, 54, NULL, trackdesc },		// 11 Tracks
	{ roadstvars, 110, roadstaux, roadstdesc },	// 12 Road stations
	{ trkmodvars, 18, NULL, simpledesc },		// 13 Road modifications
	{ roadvars, 48, NULL, roaddesc },		// 14 Roads
	{ airportvars, 186, airportaux, airportdesc },	// 15 Airports
	{ dockvars, 40, dockaux, dockdesc },		// 16 Docks
	{ vehvars, 350, NULL, vehdesc },		// 17 Vehicles
	{ treevars, 76, NULL, simpledesc },		// 18 Trees
	{ simplevars, 6, NULL, simpledesc },		// 19 Snow
	{ climvars, 10, NULL, simpledescnosprites },	// 1A Climates
	{ shapevars, 14, NULL, simpledesc },		// 1B Shapes
	{ bldngvars, 190, bldngaux, bldngdesc },	// 1C bldngs
	{ scaffvars, 18, NULL, simpledesc },		// 1D Scaffolding
	{ indvars, 244, indaux, inddesc },		// 1E Industries
	{ regionvars, 18, NULL, regiondesc },		// 1F Regions
	{ compvars, 56, NULL, compdesc },		// 20 Companies
	{ simplevars, 6, NULL, textdesc },		// 21 Texts
};
